-- This is the graphics script. Do not touch this unless you know what you're doing.

local NAME=... -- Boilerplate code

local G = { }


function color(R, G, B) -- Return a table with the R, G, and B values in it
return {R, G, B}
end

function print(inStr, X, Y) -- Print inStr at X, Y
    love.graphics.setColor(1,1,1)
    love.graphics.print(inStr, X, Y)
    return 0
end

function clear(_RGB) -- Fill the screen with color _RGB
    R = _RGB[1] / 255 -- Calculate RGB values
    G = _RGB[2] / 255
    B = _RGB[3] / 255

    love.graphics.clear(R,G,B) -- Fill the screen with the R, G, and B values in the _RGB table
    return 0
end

function drawRect(x1, y1, x2, y2, _RGB) -- Draw a rectangle of color _RGB

    R = _RGB[1] / 255 -- Calculate RGB values
    G = _RGB[2] / 255
    B = _RGB[3] / 255


end

function drawEllipse(x, y, width, height)
    love.graphics.ellipse("empty", x, y, width, height)
end

function drawEllipseF(x, y, width, height)
    love.graphics.ellipse("fill", x, y, width, height)
end

function drawLine(x, y, x2, y2, lineColor)
    R = lineColor[1] / 255 -- Calculate RGB values
    G = lineColor[2] / 255
    B = lineColor[3] / 255

    love.graphics.setColor(R, G, B)
    love.graphics.line(x, y, x2, y2)
end

function drawBox3D(xOffset, yOffset, renderMode, points, lines, pointSize, pointColor, lineColor, faceColor) -- Draws a 3D box
    pR = pointColor[1] / 255 -- Calculate RGB values for pointColor
    pG = pointColor[2] / 255
    pB = pointColor[3] / 255

    lR = lineColor[1] / 255 -- Calculate RGB values for lineColor
    lG = lineColor[2] / 255
    lB = lineColor[3] / 255

    fR = faceColor[1] / 255 -- Calculate RGB values for faceColor
    fG = faceColor[2] / 255
    fB = faceColor[3] / 255

    if (renderMode == "onlyPoints" or renderMode == "wireframe" or renderMode == "facesAndPoints" or renderMode == "e")
    then
        for i=1,#points do -- Render points with color pointColor
            point = points[i]
            --switch (i)
            --case 1:
            --love.graphics.setColor(pR, pG, pB)
            --break
            --case 2:
            --love.graphics.setColor(pR, pG, pB)
            love.graphics.setColor(pR, pG, pB)
            drawEllipseF(point[1] + xOffset, point[2] + yOffset, pointSize, pointSize)
        end
    end
    if (renderMode == "faces" or renderMode == "facesAndPoints" or renderMode == "e")
    then
        for i=1,#lines do
            p1 = lines[i][1]
            p2 = lines[i][2]
            p3 = lines[i][3]
            point1 = points[p1]
            point2 = points[p2]
            point3 = points[p3]
            if (i == 1) -- Sets a color for indavidual faces
            then
                love.graphics.setColor(1,0,0)
            end
            if (i == 2)
            then
                love.graphics.setColor(1,0,0)
            end
            if (i == 3)
            then
                love.graphics.setColor(0,1,0)
            end
            if (i == 4)
            then
                love.graphics.setColor(0,1,0)
            end
            if (i == 5)
            then
                love.graphics.setColor(0,1,0)
            end
            
            love.graphics.polygon('fill', point1[1] + xOffset, point1[2] + yOffset, point2[1] + xOffset, point2[2] + yOffset, point3[1] + xOffset, point3[2] + yOffset)
        end
    end
    -- TODO: Add shading using basic raycasting

    if (renderMode == "onlyLines" or renderMode == "wireframe" or renderMode == "facesAndLines")
    then
        for i=1,#lines do -- Render lines with color lineColor
            p0 = lines[i][1]
            p1 = lines[i][2]
            point0 = points[p0]
            point1 = points[p1]
            love.graphics.setColor(lR, lG, lB)
            love.graphics.line(point0[1] + xOffset, point0[2] + yOffset, point1[1] + xOffset, point1[2] + yOffset)
        end
    end
    if (renderMode == "polygonLines" or renderMode == "e")
    then
        for i=1,#lines do
            p1 = lines[i][1]
            p2 = lines[i][2]
            p3 = lines[i][3]
            point1 = points[p1]
            point2 = points[p2]
            point3 = points[p3]
            love.graphics.setColor(lR, lG, lB)
            love.graphics.polygon('line', point1[1] + xOffset, point1[2] + yOffset, point2[1] + xOffset, point2[2] + yOffset, point3[1] + xOffset, point3[2] + yOffset)
        end
    end
end

function rotPointsZ(theta, points) -- Rotate the points in the Z direction by theta degrees
    sinTheta = math.sin(theta)
    cosTheta = math.cos(theta)
    rotatedPoints = {}
        for i=1,#points do
            point = points[i]
            x = point[1]
            y = point[2]
            z = point[3]
            calcPointX = x * cosTheta - y * sinTheta
            calcPointY = y * cosTheta + x * sinTheta
            rotatedPoints[i] = {calcPointX, calcPointY, z}
        end
    return rotatedPoints
end

function rotPointsX(theta, points) -- Rotate the points in the X direction by theta degrees
    sinTheta = math.sin(theta)
    cosTheta = math.cos(theta)
    rotatedPoints = {}
        for i=1,#points do
            point = points[i]
            x = point[1]
            y = point[2]
            z = point[3]
            calcPointY = y * cosTheta - z * sinTheta
            calcPointZ = z * cosTheta + y * sinTheta
            rotatedPoints[i] = {x, calcPointY, calcPointZ}
        end
    return rotatedPoints
end

function rotPointsY(theta, points)
    sinTheta = math.sin(theta)
    cosTheta = math.cos(theta)
    rotatedPoints = {}
        for i=1,#points do
            point = points[i]
            x = point[1]
            y = point[2]
            z = point[3]
            calcPointX = x * cosTheta + z * sinTheta
            calcPointZ = z * cosTheta - x * sinTheta
            rotatedPoints[i] = {calcPointX, y, calcPointZ}
        end
    return rotatedPoints
end

G.color = color -- More boilerplate code
G.print = print
G.clear = clear
G.drawRect = drawRect
G.drawEllipse = drawEllipse
G.drawEllipseF = drawEllipseF
return G